package de.desy.mst.libs.ser.tclientlib.readers;
import de.desy.tine.dataUtils.TDataType;
import de.desy.mst.libs.ser.tclientlib.ATineReader;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.mst.libs.ser.tclientlib.TLinkAddress;
import de.desy.tine.types.SPECTRUM;


/**
 * A SPECTRUM reader.
 *
 * The SPECTRUMReader reads a scalar value of type SPECTRUM.
 *
 */
public class SPECTRUMReader extends ATineReader {
  
    private static final int DEFAULT_DATA_LENGTH = 4096;
    private SPECTRUM inputBuffer = null;; // Data buffer for TINE
    private SPECTRUM outputBuffer = null; // Data buffer for TINE
	private int dataLength = 4096;
    
    /**
     * Constructor.
     * @param linkAddress The TINE address
     */
    public SPECTRUMReader(TLinkAddress linkAddress, int dataLength) {
        super(linkAddress);
        this.dataLength = dataLength;
        createBuffers();
        setup(createTDataType());
    }

    /**
     * Constructor.
     * @param linkAddress The TINE address
     */
    public SPECTRUMReader(TLinkAddress linkAddress) {
        super(linkAddress);
        this.dataLength = DEFAULT_DATA_LENGTH;
        createBuffers();
        setup(createTDataType());
     }

    
    /**
     * Constructor.
     * @param fullDevicePath The full TINE path of
     * the property. It must be in the form 
     * <code>
     * /context/server/device/property
     * </code>.
     */
    public SPECTRUMReader(String fullDevicePath) {
        super(fullDevicePath);
        this.dataLength = DEFAULT_DATA_LENGTH;
        createBuffers();
        setup(createTDataType());
    }

    /**
     * Returns the value.
     * If the reader is in asynchronous mode (attached), 
     * the last updated values are returned. If the reader
     * in in synchronous mode, a synchronous read operation
     * (execute) is performed.
     * In both cases, when an error occurred, no valid data
     * are available, no value is returned,
     * and a TineLinkException is thrown instead.
     * @throws TineLinkException
     */
    public synchronized SPECTRUM getValue() throws TineLinkException {
        if (isSynchronous()) {
            readData();
            copyData();
        }
        if (hasError()) {
            throw new TineLinkException(getLinkAddress(), getLinkStatus(), getErrorMessage());
        } 
        return outputBuffer;
    }
    
    
    /**
     * Copies the input array to the output array.
     * This method is called when the reader is in asynchronous mode.
     */
     protected synchronized void copyData() {
           outputBuffer = new SPECTRUM(inputBuffer);
    }
 

    private void createBuffers() {
        inputBuffer = new SPECTRUM(dataLength);
        copyData();
    }
    
    private TDataType createTDataType() {
        TDataType result = null;
        result = new TDataType(inputBuffer);
        return result;	
    }
    
}
